denIXargc <-
function(dser,juvper=50,initper=10,missing=NA,gam=F,thresholdR2=NA,outfile=NULL)
{
# initialisation
	if(gam==FALSE) {require(nnet)}
	if(gam==TRUE) {require(mgcv)}
	n<-nrow(dser)
	m<-ncol(dser)
	dser[dser==missing]=NA
  print(sum(is.na(dser)))
	IXargc<-list(title="Tree-ring series indexing using ARGC method")
	if (gam==TRUE) {IXargc$method="GAM"} else {IXargc$method="ANN"}
	IXargc$initper<-initper
	IXargc$juvper<-juvper
	IXargc$R2<-rep(0,m)
	IXargc$treetrend<-matrix(NA,n,m)
	IXargc$treeindex<-matrix(NA,n,m)
	if (!is.null(outfile)) {
		fgraph=paste(outfile,".pdf",sep="")
		ftext=paste(outfile,".txt",sep="")
		sink(file=ftext)
		pdf(file=fgraph)
	}

	cat("Dendro indexation using ARGC \n")
	cat("----------------------------------------------------------\n")

# loop on tree-ring series
	time<-as.numeric(rownames(dser))
	imin=time[1]
	xm=matrix(NA,m,4)
	age=matrix(NA,m,2)
	statab=0
	cat('Tree  -   starting growth - max juv. growth - starting & ending date - Mean/std.dev (of smoothed series) - juvper\n')
	for (j in 1:m)
	{
		data<-data.frame(time=time,trser=dser[,j])
		trend<-loess(trser ~ time,data,span=0.1)
# a few statistics for each tree
		jp=min(juvper,length(trend$fitted))
		xm[j,2]=max(trend$fitted[1:jp])
		xm[j,1]=mean(trend$fitted[1:initper])
		xm[j,3]=mean(trend$fitted)
		xm[j,4]=sd(trend$fitted)
		age[j,1]=trend$x[1]
		nt<-trend$n
		age[j,2]=trend$x[nt]
		cat(colnames(dser)[j],xm[j,1:2],age[j,1:2],xm[j,3:4],jp,"\n")
		tmp=matrix(0,nt,5)
# predictors
		tmp[1:nt,1]=trend$x-age[j,1]
		tmp[1:nt,2]=rep(xm[j,1],nt)
		tmp[1:nt,3]=rep(xm[j,2],nt)
# ring width series
		tmp[1:nt,4]=trend$y
# smoothed ring width series used for spin up
		tmp[1:nt,5]=trend$fitted
		if (j>1) {statab=rbind(statab,tmp[1:nt,])}
		else {statab=tmp}
	}

# calibration ANN
	if (gam==F) {
		colnames(statab)<-c("Age","Initgr","Maxgr","ring","Sring")
		stmin<-apply(statab,2,min)
		stmax<-apply(statab,2,max)
		st<-scale(statab,stmin,stmax-stmin)
# spin up
		print(paste(nrow(st),sum(is.na(st))))
		st.nn <- nnet(st[,1:3], st[,5], size = 4,maxit = 1000,entropy=TRUE)
		plot(c(0,1),c(0,1), type='l',xlab='obs', ylab='est', main='trend estimation by ANN (spinup using smoothed data)')
		points(st[,5],st.nn$fitted.values,col="red")
# final calibration
		st.nn <- nnet(st[,1:3], st[,4], size = 4, maxit=1000 ,Wts=st.nn$wts)
		plot(c(0,1),c(0,1),type='l',xlab='obs',ylab='est', main='trend estimation by ANN (final estimates)')
		points(st[,4],st.nn$fitted.values,col="red")
	} else {

# calibration GAM
		vd=st[,4]
		x1=st[,1]
		x2=st[,2]
		x3=st[,3]
		gamm<-gam(vd~s(x1)+s(x2)+s(x3))
		print(summary(gamm))
		print(anova(gamm))
	}
# standardization
	layout(matrix(1:2,2,1))
	for (j in 1:m)
	{
		cat("\nTree-ring series : ",colnames(dser)[j],"\n")
		cat("--------------------------------------------\n")
		nt=age[j,2]-age[j,1]+1
		tmp=matrix(0,nt,3)
		tmp[,1]=(0:(nt-1)-stmin[1])/(stmax[1]-stmin[1])
		tmp[,2]=(rep(xm[j,1],nt)-stmin[2])/(stmax[2]-stmin[2])
		tmp[,3]=(rep(xm[j,2],nt)-stmin[3])/(stmax[3]-stmin[3])
		colnames(tmp)<-c("Age","Initgr","Maxgr")
		if (gam==F) {
			z=predict(st.nn,newdata=tmp,type="raw")
			z=z*(stmax[4]-stmin[4])+stmin[4]
		} else {
			mat=data.frame(x1=0:(nt-1),x2=rep(xm[j,1],nt),x3=rep(xm[j,2],nt))
			z<-predict(gamm,mat)
		}
		i1=age[j,1]-imin+1
		i2=age[j,2]-imin+1
		IXargc$treetrend[i1:i2,j]=z
		IXargc$treeindex[i1:i2,j]=dser[i1:i2,j]/z
		IXargc$R2[j]=1-mean((dser[,j]-IXargc$treetrend[,j])^2,na.rm=TRUE)/var(dser[,j],na.rm=TRUE)
		cat(paste("R2 = ",IXargc$R2[j],"\n"))
		cat(paste("Raw mean = ",mean(dser[,j],na.rm=TRUE),"\n"))
		cat(paste("Trend mean = ",mean(IXargc$treetrend[,j],na.rm=TRUE),"\n"))
		plot(time,dser[,j], type='l',xlab="time", ylab='tree-ring', main=paste("Tree",colnames(dser)[j]))
		lines(time,IXargc$treetrend[,j], col='red')
		plot(time,IXargc$treeindex[,j], type='l',xlab="time", ylab='tree-index', main=paste("Tree",colnames(dser)[j]))
		lines(c(time[1],time[n]),c(1,1),col="blue")
	}
	rownames(IXargc$treetrend)<-rownames(dser)
	rownames(IXargc$treeindex)<-rownames(dser)
	colnames(IXargc$treetrend)<-colnames(dser)
	colnames(IXargc$treeindex)<-colnames(dser)

# grouping by pop
  TR2=thresholdR2
  if (is.na(TR2)) {TR2=-9999}
	treebelong<-substr(colnames(dser),1,nchar(colnames(dser))-2)
  treebelong[which(IXargc$R2<TR2)] <- "BADTREES"
  if (!is.na(thresholdR2)) {
    cat("Threshold used for R2 (trees with R2 below this value are removed) = ",thresholdR2,"\n")
    cat("Number of rejected trees = ",sum(treebelong=="BADTREES"),"\n")
    cat("Names of the rejected trees\n")
    print(colnames(dser)[which(IXargc$R2<TR2)])
  }
	popnames<-unique(treebelong)
	z1<-data.frame(t(IXargc$treeindex))
  z2<-data.frame(!is.na(z1))
	tree<-factor(treebelong,levels=popnames)
	zz1<-aggregate(z1,list(Pop=treebelong),mean,na.rm=TRUE)
	zz2<-aggregate(z2,list(Pop=treebelong),sum,na.rm=TRUE)
	zz3<-aggregate(z1,list(Pop=treebelong),sd,na.rm=TRUE)
	mt<-nrow(zz1)
	IXargc$popindex<-matrix(NA,n,mt)
	IXargc$popise<-matrix(NA,n,mt)
	IXargc$popsize<-matrix(NA,n,mt)
	IXargc$varpop<-matrix(NA,mt,2)
	layout(matrix(1:2,2,1))
  popnames<-zz1[,1]
	for (j in 1:mt)
	{
		IXargc$popindex[1:n,j]=as.matrix(zz1[j,2:(n+1)])
		IXargc$popise[1:n,j]=as.matrix(zz3[j,2:(n+1)])
		IXargc$popsize[1:n,j]=as.matrix(zz2[j,2:(n+1)])
		IXargc$popise=IXargc$popise/IXargc$popsize^0.5
		plot(time,IXargc$popindex[,j], type='l',xlab="time", ylab='pop index', main=paste("Population",popnames[j]))
		lines(c(time[1],time[n]),c(1,1),col="blue")
		plot(time,IXargc$popsize[,j],type='s',xlab='time', ylab='pop size', main=paste("Population",popnames[j]))
		IXargc$varpop[j,1]=mean(IXargc$popindex[1:n,j],na.rm=TRUE)
		IXargc$varpop[j,2]=var(IXargc$popindex[1:n,j],na.rm=TRUE)
	}
	colnames(IXargc$popindex)<-popnames
	rownames(IXargc$popindex)<-rownames(dser)
	colnames(IXargc$popise)<-popnames
	rownames(IXargc$popise)<-rownames(dser)
	colnames(IXargc$popsize)<-popnames
	rownames(IXargc$popsize)<-rownames(dser)	
	rownames(IXargc$varpop)<-popnames
	colnames(IXargc$varpop)<-c("Mean_on_year","Var_on_year")

  if (!is.null(outfile)) {
		cat("Tree Trends\n",file=ftext,append=TRUE)
		write.table(IXargc$treetrend,file=ftext,append=TRUE)
		cat("Tree Indices\n",file=ftext,append=TRUE)
		write.table(IXargc$treeindex,file=ftext,append=TRUE)
		cat("Population Indices\n",file=ftext,append=TRUE)
		write.table(IXargc$popindex,file=ftext,append=TRUE)
		cat("Population Indices SE\n",file=ftext,append=TRUE)
		write.table(IXargc$popise,file=ftext,append=TRUE)
		cat("Population Sizes\n",file=ftext,append=TRUE)
		write.table(IXargc$popsize,file=ftext,append=TRUE)
		cat("Population variances\n",file=ftext,append=TRUE)
		write.table(IXargc$varpop,file=ftext,append=TRUE)
		sink(file=ftext,append=TRUE)
		dev.off()
		sink()
		sink()
		sink()
	}

	return(IXargc)
}
