denIXsm <-
function(dser,window=0.5,missing=NA,outfile=NULL)
{
# initialisation
#	library(gstat)
	n<-nrow(dser)
	m<-ncol(dser)
	dser[dser==missing]=NA
	if (!is.null(outfile)) {
		fgraph=paste(outfile,".pdf",sep="")
		ftext=paste(outfile,".txt",sep="")
		sink(file=ftext)
		pdf(file=fgraph)
	}

	if (window>1) {
		cat("Dendro indexation using Fourier band pass filter \n")
		cat("---------------------------------------------------------------\n")
		window=min(10000,window)
		cat("\nWindow size (years) : ",window,"\n")
		IXsm<-list(title="Tree-ring series indexing using BPfilter (fourier band pass) method")
	} else {
		cat("Dendro indexation using local regression filter \n")
		cat("---------------------------------------------------------------\n")
		window=max(0.001,window)
		cat("\nWindow proportion : ",window,"\n")
		IXsm<-list(title="Tree-ring series indexing using LOESS method")
		}
	IXsm$R2<-rep(0,m)
	IXsm$window<-window
	IXsm$cortrend<-matrix(NA,n,m)
	IXsm$corindex<-matrix(NA,n,m)
# loop on tree-ring series
	time<-as.numeric(rownames(dser))
	imin=time[1]-1
	for (j in 1:m)
	{
		cat("\nTree-ring series : ",colnames(dser[j]),"\n")
		cat("----------------------------------------\n")
		data<-data.frame(time=time,trser=dser[,j])
		layout(matrix(1:2,2,1))
		plot(time,dser[,j], type='l',xlab="time", ylab='ring width', main=colnames(dser)[j])
		if (window>1) {
			trend=BPfilter(dser[,j],fband=c(0,1/window))
			IXsm$cortrend[time-imin,j]=trend
			IXsm$corindex[time-imin,j]=dser[,j]/trend
			lines(time,trend,col="red")
			plot(time,IXsm$corindex[,j], type='l',xlab="time", ylab='index', main=colnames(dser)[j])
		} else {
			trend<-loess(trser ~ time,data,span=window)
			print(summary(trend))
			IXsm$cortrend[trend$x-imin,j]=trend$fitted
			IXsm$corindex[trend$x-imin,j]=trend$y/trend$fitted
			lines(trend$x,trend$fitted,col="red")
			plot(time,IXsm$corindex[,j], type='l',xlab="time", ylab='index', main=colnames(dser)[j])
			trend=trend$fitted
		}
		IXsm$R2[j] <- var(trend,na.rm=T)/var(dser[,j],na.rm=TRUE)
		print(paste("R2 = ",IXsm$R2[j]))
	}
	rownames(IXsm$cortrend)<-rownames(dser)
	rownames(IXsm$corindex)<-rownames(dser)
	colnames(IXsm$cortrend)<-colnames(dser)
	colnames(IXsm$corindex)<-colnames(dser)

# grouping by tree
	corbelong<-substr(colnames(dser),1,nchar(colnames(dser))-1)
	treenames<-unique(corbelong)
	z<-data.frame(t(IXsm$corindex))
	tree<-factor(corbelong,levels=treenames)
	zz<-aggregate(z,list(Tree=corbelong),mean,na.rm=TRUE)
	mt<-nrow(zz)
	IXsm$trindex<-matrix(NA,n,mt)
	layout(matrix(1:2,2,1))
	for (j in 1:mt)
	{
		IXsm$trindex[1:n,j]=as.matrix(zz[j,2:(n+1)])
		plot(time,IXsm$trindex[,j], type='l',xlab="time", ylab='tree index', main=paste("Tree",treenames[j]))
	}
	colnames(IXsm$trindex)<-treenames
	rownames(IXsm$trindex)<-rownames(dser)		

# global mean
	layout(matrix(1:2,2,1))
	IXsm$popindex<-apply(IXsm$trindex,1,mean,na.rm=TRUE)
	IXsm$popsize<-apply(!is.na(dser),1,sum)
	popname<-substr(treenames[1],1,nchar(treenames)-2)
	plot(time,IXsm$popindex,type='l',xlab='time', ylab='pop index', main=paste("Mean population",popname))
	plot(time,IXsm$popsize,type='s',xlab='time', ylab='pop size', main=paste("Mean population",popname))
	IXsm$popindex=as.matrix(IXsm$popindex)
	IXsm$popsize=as.matrix(IXsm$popsize)
	colnames(IXsm$popindex)<-popname
	rownames(IXsm$popindex)<-rownames(dser)		
	colnames(IXsm$popsize)<-popname
	rownames(IXsm$popsize)<-rownames(dser)
	if (!is.null(outfile)) {
		cat("Core trends\n",file=ftext,append=TRUE)
		write.table(IXsm$cortrend,file=ftext,append=TRUE)
		cat("Core Indices\n",file=ftext,append=TRUE)
		write.table(IXsm$corindex,file=ftext,append=TRUE)
		cat("Tree Indices\n",file=ftext,append=TRUE)
		write.table(IXsm$trindex,file=ftext,append=TRUE)
		cat("Population Indices\n",file=ftext,append=TRUE)
		write.table(IXsm$popindex,file=ftext,append=TRUE)
		cat("Population Sizes\n",file=ftext,append=TRUE)
		write.table(IXsm$popsize,file=ftext,append=TRUE)
		dev.off()
		sink()
		sink()
		sink()
	}

	return(IXsm)
}
