denRFcpb <-
function(xmat,ymat,p=0,ncas=100,outfile=NULL)
{
# Initialisation
nx<-nrow(xmat)
mx<-ncol(xmat)
ny<-nrow(ymat)
my<-ncol(ymat)
if (!is.null(outfile)) 
	{
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
	}
if (identical(nx,ny)==FALSE) { stop ("both xmat & ymat matrices have not same number of rows") }
RFcpb<-list(title="response function using a bootstrapped regression on CP")
RFcpb$r2<-rep(0,my)
RFcpb$rmse<-rep(0,my)
RFcpb$ymat<-array(0,c(ny,my*3))
date=as.numeric(rownames(ymat))
errc<-rep(0,ncas)
errv<-errc
ncal<-ny*4/5
Yec<-array(0,c(ny,my,ncas))
Beta<-array(0,c(mx,my,ncas))
Ye3<-array(0,c(3,ny,my))
RFcpb$Beta3<-array(0,c(3,mx,my))
RFcpb$signif=matrix(NA,mx,my)
# standardization
ymoy<-apply(ymat,2,mean)
ysd<-apply(ymat,2,sd)
xmoy<-apply(xmat,2,mean)
xsd<-apply(xmat,2,sd)
Y<-as.matrix(scale(ymat,center=ymoy,scale=ysd))
X<-as.matrix(scale(xmat,center=xmoy,scale=xsd))
# beginning bootstrap
cat("Case boot - err.cal - err.ver\n")
for (kcas in 1:ncas) 
	{
# tirage au sort pour constituer un échantillon de calibration
	samp <- sample(1:ny,ny,replace=TRUE)
	Xcal=X[samp,]
	Ycal=Y[samp,]
# transformation in pc if p>0
	if (p>0) 
		{
		pc.X<-prcomp(Xcal,scale.=FALSE)
		Xcal=pc.X$x[,1:p]
		sc.X<-predict(pc.X,X)[,1:p]
		A=pc.X$rotation[,1:p]
		fmla<-as.formula(paste("Ycal ~ 0 +",paste(colnames(Xcal),collapse="+")))
		} 
	else 
		{
		sc.X<-X
		A=diag(1,mx,mx)
		p<-mx
		fmla<-as.formula(paste("Ycal ~ 0 +",paste(colnames(X),collapse="+")))
		}
	dd.lm <- lm(fmla,data=as.data.frame(cbind(Xcal,Ycal))) 
	Yec[,,kcas]<-as.matrix(predict(dd.lm,as.data.frame(sc.X)))
	errc[kcas]<-mean(sd(dd.lm$residuals[samp])^2)
	errv[kcas]<-mean(sd(dd.lm$residuals[-samp])^2)
	print(c(kcas,errc[kcas],errv[kcas]),digits=3)
	Beta[,,kcas]=A%*%dd.lm$coefficients
	}
cat("RESPONSE FUNCTION : bootstrapped regression on PCs\n")
cat("--------------------------------------------------------------\n")
cat("Calibration error (median and 95%-conf. interval)\n")
RFcpb$errc=quantile(errc,probs=c(0.025, 0.5, 0.975))
print(RFcpb$errc)
cat("Verification error (median and 95%-conf. interval)\n")
RFcpb$errv=quantile(errv,probs=c(0.025, 0.5, 0.975))
print(RFcpb$errv)
Ye3<-apply(Yec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975))
RFcpb$Beta3<-apply(Beta,c(1,2),quantile,probs=c(0.025, 0.5, 0.975))
# ymat variable loop
for (j in 1:my) 
	{
	Ye3[,,j]<-Ye3[,,j]*ysd[j]+ymoy[j]
	cat("\nDependent Variable : ",colnames(ymat[j]),"\n")
	cat("----------------------------------------\n")
	RFcpb$ymat[,(j-1)*3+1]<-Ye3[2,,j]
	RFcpb$ymat[,(j-1)*3+2]<-Ye3[2,,j]-Ye3[1,,j]
	RFcpb$ymat[,(j-1)*3+3]<-Ye3[3,,j]-Ye3[2,,j]
	RFcpb$rmse[j]<-mean((ymat[,j]-Ye3[2,,j])^2)^0.5
	RFcpb$r2[j]<-1-mean((ymat[,j]-Ye3[2,,j])^2)/var(ymat[,j])
	cat("R2 = ",RFcpb$r2[j],"  -- RMSE = ",RFcpb$rmse[j],"calculated on ",ny,"  observations\n")
	cat("Regression Coefficients with 95% confidence interval and prob-value\n")
	for (k in 1:mx) 
		{
		RFcpb$signif[k,j]=mean(Beta[k,j,]>0)
		if (RFcpb$signif[k,j]<0.5) {RFcpb$signif[k,j]=1-RFcpb$signif[k,j]}
		cat(colnames(xmat)[k],format(RFcpb$Beta3[1:3,k,j],digit=3,nsmall=3)," prob= ",RFcpb$signif[k,j],"\n")
		}
# plot of the response function
	if (mx<=16) 
		{
		dotchart(RFcpb$Beta3[2,,j],labels=colnames(xmat),main=paste("Response function of",colnames(ymat)[j]),xlim=c(-1,1),xlab="Std regr coeff")
		lines(rep(0,mx),1:mx)
		points(RFcpb$Beta3[1,,j],1:mx,col="gray")
		points(RFcpb$Beta3[3,,j],1:mx,col="gray")
		} 
	else 
		{
		layout(matrix(1:2,1,2))
		mx2=mx/2
		mx1=mx/2+1
		dotchart(RFcpb$Beta3[2,1:mx2,j],labels=colnames(xmat)[1:mx2],main="Response function",xlim=c(-1,1),xlab="Std regr coeff")
		lines(rep(0,mx2),1:mx2)
		points(RFcpb$Beta3[1,1:mx2,j],1:mx2,col="gray")
		points(RFcpb$Beta3[3,1:mx2,j],1:mx2,col="gray")
		dotchart(RFcpb$Beta3[2,mx1:mx,j],labels=colnames(xmat)[mx1:mx],xlim=c(-1,1),xlab="Std regr coeff",main=colnames(ymat)[j])
		lines(rep(0,mx2),1:mx2)
		points(RFcpb$Beta3[1,mx1:mx,j],1:mx2,col="gray")
		points(RFcpb$Beta3[3,mx1:mx,j],1:mx2,col="gray")
		}
# plot of the observed/estimated growth
	layout(matrix(1:1,1,1))
	plot(date,ymat[,j], type='l',main=colnames(ymat)[j], ylab="Obs / Est", xlab="Obs (red)/ Est (black) w/ 95% interval")
	polygon(c(date,rev(date)),c(Ye3[1,,j],rev(Ye3[3,,j])),col="gray",border=FALSE)
	lines(date,RFcpb$ymat[,(j-1)*3+1],col='Black')
	lines(date,ymat[,j],col='red')
	}
# end of output
coln<-rep(NA,my*3)
for (j in 1:my) 
	{
	coln[(j-1)*3+1]<-colnames(ymat)[j]
	coln[(j-1)*3+2]<-paste(colnames(ymat)[j],'_i',sep="")
	coln[(j-1)*3+3]<-paste(colnames(ymat)[j],'_s',sep="")
	}
colnames(RFcpb$ymat)<-coln
colnames(RFcpb$Beta3)<-colnames(xmat)
rownames(RFcpb$ymat)<-rownames(ymat)	
if (!is.null(outfile)) 
	{
	sink(file=ftext,append=TRUE)
	cat("\nImpression de la liste RFcpb\n")
#	print(IXargc)
	print(RFcpb)
	dev.off()
	sink()
	sink()
	sink()
	
	}	
return(RFcpb)
}
