predLMb <-
function(xmat,ymat,xnewmat,ynewmat=FALSE,p=0,ncas=100,xstep=1,outfile=NULL)
{
# Initialisation
if (!is.null(outfile)) {
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
}

na<-nrow(xmat)
ma<-ncol(xmat)
nc<-nrow(ymat)
mc<-ncol(ymat)
nfa<-nrow(xnewmat)
mfa<-ncol(xnewmat)
if (identical(na,nc)==FALSE) {stop ("both xmat & ymat matrices have not same number of rows")}
if (identical(mfa,ma)==FALSE) {stop ("both xmat & xnewmat matrices have not same number of columns")}
nfc<-0
mfc<-0
option="extrapolation"
LMB<-list(title="Linear model estimates by bootstrap method")
LMB$rmseb<-matrix(0,mc,3)
LMB$r2b<-matrix(0,mc,3)
LMB$ymat<-array(0,c(nc,mc*3))
LMB$ynewmat<-array(0,c(nfa,mc*3))
LMB$rmsepb<-matrix(0,mc,3)
LMB$reb<-matrix(0,mc,3)
LMB$ceb<-matrix(0,mc,3)
if (is.logical(ynewmat)==FALSE) {
	nfc<-nrow(ynewmat)
	mfc<-ncol(ynewmat)
	if (identical(nfc,nfa)==FALSE) stop ("both ynewmat & xnewmat matrices have not same number of rows")
	option="verification"
	LMB$rmsep<-rep(0,mc)
	LMB$r2p<-rep(0,mc)
	LMB$CE<-rep(0,mc)

}

errc<-rep(0,ncas)
errv<-errc
ncal<-na*4/5
Yec<-array(0,c(nc,mc,ncas))
Zec<-array(0,c(nfa,mc,ncas))
Ye3<-array(0,c(3,nc,mc))
Ze3<-array(0,c(3,nfa,mc))
r2=matrix(NA,mc,ncas)
re=matrix(NA,mc,ncas)
ce=matrix(NA,mc,ncas)
rmsep=matrix(NA,mc,ncas)
rmse=matrix(NA,mc,ncas)

cat("REGRESSION ON PRINCIPAL COMPONENTS AND BOOTSTRAPPING\n")
cat("--------------------------------------------------------------\n")
cat("\nNb PC's : ",p,"\n")
cat("\nNb bootstrap simulations : ",ncas,"\n")
cat("\nStep for resampling : ",xstep,"\n")

# transformation in pc if p>0

if (p>0) {
	pc.xmat<-prcomp(xmat,scale.=TRUE)
	screeplot(pc.xmat, npcs=p, type="barplot",main="Screeplot of the eignevalues")
	biplot(pc.xmat)
	z.xmat<-predict(pc.xmat,xmat)
	z.xnewmat<-predict(pc.xmat,xnewmat)
	print(summary(pc.xmat))
} else {
	z.xmat<-xmat
	z.xnewmat<-xnewmat
	p<-ma
}

# standardization

ymoy<-apply(ymat,2,mean)
ysd<-apply(ymat,2,sd)
Y<-as.matrix(scale(ymat,center=ymoy,scale=ysd))
X<-as.matrix(z.xmat[,1:p])
Z<-as.matrix(z.xnewmat[,1:p])

# estimation on all the data
fmla<-as.formula(paste("Y~",paste(colnames(X),collapse="+")))
dd.lm <- lm(fmla,data=data.frame(cbind(X,Y))) 
Ye<-as.matrix(predict(dd.lm,newdata=as.data.frame(X)))
err<-mean(sd(dd.lm$residuals)^2/sd(Y)^2)
print(dd.lm)

# beginning bootstrap
if(ncas>0) {
	cat("Case boot - err.cal - err.ver\n")
	for (kcas in 1:ncas) {

# tirage au sort pour constituer un echantillon de calibration
		xlist=seq(1,na,by=xstep)
		samp <- sample(xlist,na,replace=T)
		Xcal=as.matrix(X[samp,]); rownames(Xcal)=1:length(samp)
		Ycal=as.matrix(Y[samp,]); rownames(Ycal)=1:length(samp)
		fmlaa<-as.formula("Ycal~Xcal")
		dd.lm <- lm(fmlaa,data=data.frame(Xcal=Xcal,Ycal=Ycal)) 
#		Yec[,,kcas]<-as.matrix(predict(dd.lm,new.data=X))
		Yec[,,kcas]<-as.matrix(cbind(rep(1,nrow(X)),X)) %*% as.matrix(dd.lm$coefficients)
		Zec[,,kcas]<-as.matrix(cbind(rep(1,nrow(Z)),Z)) %*% as.matrix(dd.lm$coefficients)
#		Zec[,,kcas]<-as.matrix(predict(dd.lm,new.data=data.frame(Xcal=Z)))
		errc[kcas]<-mean(sd(dd.lm$residuals[samp])^2)
		errv[kcas]<-mean(sd(dd.lm$residuals[-samp])^2)
		print(c(kcas,errc[kcas],errv[kcas]),digits=3)
		for (j in 1:mc) {
			rmse[j,kcas]=ysd[j]*(mean((Y[samp,j]-Yec[samp,j,kcas])^2))^0.5
			r2[j,kcas]=1-mean((Y[samp,j]-Yec[samp,j,kcas])^2)/var(Y[samp,j])
			rmsep[j,kcas]=ysd[j]*(mean((Y[-samp,j]-Yec[-samp,j,kcas])^2))^0.5
			re[j,kcas]=1-mean((Y[-samp,j]-Yec[-samp,j,kcas])^2)/mean((Y[-samp,j]-mean(Y[samp,j]))^2)
			ce[j,kcas]=1-mean((Y[-samp,j]-Yec[-samp,j,kcas])^2)/mean((Y[-samp,j]-mean(Y[-samp,j]))^2)
		}	
	}

	cat("Calibration error (median and 95%-conf. interval)\n")
	print(quantile(errc,probs=c(0.025, 0.5, 0.975)))
	cat("Verification error (median and 95%-conf. interval)\n")
	print(quantile(errv,probs=c(0.025, 0.5, 0.975)))

# ymat variable loop

cat("In the verification step, r2p (or reduction of error) compares the predictions to the calibration period ymatatology\n") 
cat("In the verification step, CE (or coefficient of efficiency) compares the predictions to the verification period ymatatology\n") 

	Ye3<-apply(Yec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975))
	Ze3<-apply(Zec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975))
	for (j in 1:mc) {

# calibration
		Ye3[,,j]<-Ye3[,,j]*ysd[j]+ymoy[j]
		cat("\nDependent Variable : ",colnames(ymat[j]),"\n")
		cat("----------------------------------------\n")
		LMB$ymat[,(j-1)*3+1]<-Ye3[2,,j]	
		cat("BOOTSTRAP SUMMARY\n")
		LMB$rmseb[j,]=quantile(rmse[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
		cat("bootstrap rmse (median and 95%-conf. interval): ",LMB$rmseb[j,],"\n")
		LMB$rmsepb[j,]=quantile(rmsep[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
		cat("bootstrap rmsep (median and 95%-conf. interval): ",LMB$rmsepb[j,],"\n")
		LMB$r2b[j,]=quantile(r2[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
		cat("bootstrap r2 (median and 95%-conf. interval): ",LMB$r2b[j,],"\n")
		LMB$reb[j,]=quantile(re[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
		cat("bootstrap re (median and 95%-conf. interval): ",LMB$reb[j,],"\n")
		LMB$ceb[j,]=quantile(ce[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
		cat("bootstrap ce (median and 95%-conf. interval): ",LMB$ceb[j,],"\n")
		LMB$ymat[,(j-1)*3+2]<-Ye3[2,,j]-Ye3[1,,j]
		LMB$ymat[,(j-1)*3+3]<-Ye3[3,,j]-Ye3[2,,j]
		LMB$rmse[j]<-mean((ymat[,j]-Ye3[2,,j])^2)^0.5
		smin=min(ymat[,j])
		smax=max(ymat[,j])
		plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(ymat)[j],'calib'), ylab="Predicted ANN", main="",col="green")
		points(ymat[,j],LMB$ymat[,(j-1)*3+1],pch=19,col='blue')

# verification

		if (option=="verification")
			{
			Ze3[,,j]<-Ze3[,,j]*ysd[j]+ymoy[j]
			LMB$rmsep[j]<-mean((ynewmat[,j]-Ze3[2,,j])^2)^0.5
			LMB$r2p[j]<-1-mean((ynewmat[,j]-Ze3[2,,j])^2)/var(ynewmat[,j])
			LMB$CE[j]<-1-mean((ynewmat[,j]-Ze3[2,,j])^2)/var(ymat[,j])
			cat("R2p (RE) = ",LMB$r2p[j],"  -- CE = ",LMB$CE[j],"  -- RMSEp = ",LMB$rmsep[j],"calculated on ",nfa,"  observations\n")
			LMB$ynewmat[,(j-1)*3+1]<-Ze3[2,,j]
			LMB$ynewmat[,(j-1)*3+2]<-Ze3[2,,j]-Ze3[1,,j]
			LMB$ynewmat[,(j-1)*3+3]<-Ze3[3,,j]-Ze3[2,,j]
			smin<-min(cbind(t(ymat[,j]),t(ynewmat[,j])))
			smax<-max(cbind(t(ymat[,j]),t(ynewmat[,j])))
			plot(c(smin,smax),c(smin,smax), type='l',xlab=colnames(ymat)[j], ylab="Predicted LMB", main="",col="green")
			points(ymat[,j],LMB$ymat[,(j-1)*3+1],pch=19,col='blue')
			points(ynewmat[,j],LMB$ynewmat[,(j-1)*3+1],col='red',pch=21)
			}
	else
			{

# extrapolation
			cat("the estimated ynewmat is put in the LMB$ynewmat matrix\n")
			Ye3[,,j]<-Ye3[,,j]*ysd[j]+ymoy[j]
			LMB$ynewmat[,(j-1)*3+1]<-Ze3[2,,j]
			LMB$ynewmat[,(j-1)*3+2]<-Ze3[2,,j]-Ze3[1,,j]
			LMB$ynewmat[,(j-1)*3+3]<-Ze3[3,,j]-Ze3[2,,j]
			}
		}

	coln<-rep(NA,mc*3)
	for (j in 1:mc) {
		coln[(j-1)*3+1]<-colnames(ymat)[j]
		coln[(j-1)*3+2]<-paste(colnames(ymat)[j],'_i',sep="")
		coln[(j-1)*3+3]<-paste(colnames(ymat)[j],'_s',sep="")
	}
	colnames(LMB$ymat)<-coln
	colnames(LMB$ynewmat)<-coln
	rownames(LMB$ymat)<-rownames(ymat)
	rownames(LMB$ynewmat)<-rownames(xnewmat)	
	cat("\nImpression de la liste LMB\n")
	print(LMB)
}
if (!is.null(outfile)) {
	dev.off()
	sink()
	sink()
	sink()
}

return(LMB)
}
