recANNb <-
function(ass,clim,fass,fclim=FALSE,p=0,k1,ntiter=2000,ncas=100,thresholderrv=NA,outfile=NULL)
{

# Initialisation
require(nnet)

na<-nrow(ass)
ma<-ncol(ass)
nc<-nrow(clim)
mc<-ncol(clim)
nfa<-nrow(fass)
mfa<-ncol(fass)
if (identical(na,nc)==F) {stop ("both ass & clim matrices have not same number of rows")}
if (identical(mfa,ma)==F) {stop ("both ass & fass matrices have not same number of columns")}
nfc<-0
mfc<-0
option="extrapolation"
ANN<-list(title="Transfer function estimates by bootstrap ANN method")
ANN$rmse<-rep(0,mc)
ANN$r2<-rep(0,mc)
ANN$r2b=matrix(0,mc,3)
ANN$reb=matrix(0,mc,3)
ANN$ceb=matrix(0,mc,3)
ANN$rmseb=matrix(0,mc,3)
ANN$rmsepb=matrix(0,mc,3)

ANN$clim<-array(0,c(nc,mc*3))
ANN$fclim<-array(0,c(nfa,mc*3))
if (is.logical(fclim)==F) {
	nfc<-nrow(fclim)
	mfc<-ncol(fclim)
	if (identical(nfc,nfa)==F) stop ("both fclim & fass matrices have not same number of rows")
	option="verification"
	ANN$rmsep<-rep(0,mc)
	ANN$r2p<-rep(0,mc)
	ANN$CE<-rep(0,mc)
}

errc<-rep(NA,ncas)
errv<-errc
ncal<-na*4/5
Yec<-array(NA,c(nc,mc,ncas))
Zec<-array(NA,c(nfa,mc,ncas))
Ye3<-array(0,c(3,nc,mc))
Ze3<-array(0,c(3,nfa,mc))
r2=matrix(NA,mc,ncas)
re=matrix(NA,mc,ncas)
ce=matrix(NA,mc,ncas)
rmsep=matrix(NA,mc,ncas)
rmse=matrix(NA,mc,ncas)

if (!is.null(outfile)) {
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
}

cat("TRANSFER FUNCTION : ANN method\n")
cat("--------------------------------------------------------------\n")
cat("\nOption : ",option,"\n")
cat("Nb PC's = ",p,"\n")
cat("Nb neurones in hidden layer = ",k1,"\n")
cat("Nb iterations = ",ntiter,"\n")
cat("Nb bootstrap simulations = ",ncas,"\n")
cat("Threshold given to verif_error for selecting the iterations = ",thresholderrv,"\n")

# transformation in pc if p>0

if (p>0) {
	pc.ass<-prcomp(ass,scale.=TRUE)
	summary(pc.ass)
	screeplot(pc.ass, npcs=p, type="barplot",main="Screeplot of the eignevalues")
	biplot(pc.ass)
	z.ass<-predict(pc.ass,ass)
	z.fass<-predict(pc.ass,fass)
	print(summary(pc.ass))
} else {
	z.ass<-ass
	z.fass<-fass
	p<-ma
}

# standardization

ymin<-apply(clim,2,min)
ymax<-apply(clim,2,max)
xmin<-apply(z.ass[,1:p],2,min)
xmax<-apply(z.ass[,1:p],2,max)
Y<-scale(clim,center=ymin,scale=(ymax-ymin))
X<-scale(z.ass[,1:p],center=xmin,scale=(xmax-xmin))
Z<-scale(z.fass[,1:p],center=xmin,scale=(xmax-xmin))

# estimation on all the data
dd.nn <- nnet(X, Y, size = k1, decay = 0, maxit = 		ntiter,trace=F) 
Ye<-as.matrix(predict(dd.nn,X,type="raw"))
err<-mean(sd(dd.nn$residuals)^2/sd(Y)^2)

# beginning bootstrap
nns=0
cat("Case boot - r2.cal - r2.ver\n")
layout(matrix(1:4,2,2))

kcas=0
thresholderrv[is.na(thresholderrv)] = -1e+10
for (kc in 1:ncas) {

# tirage au sort pour constituer un échantillon de calibration
	samp <- sample(1:na,na,replace=TRUE)
	Xcal=X[samp,]
	Ycal=Y[samp,]
	dd.nn <- nnet(Xcal, Ycal, size = k1, decay = 0.00, maxit = ntiter, trace=FALSE) 
  fv=as.matrix(predict(dd.nn,X,type="raw"))
  errvv = 1-mean(apply((Y[-samp,]-as.matrix(fv[-samp,]))^2,2,mean)/sd(Y)^2)
  if (errvv > thresholderrv) {
    kcas=kcas+1
	  Zec[,,kcas]<-as.matrix(predict(dd.nn,Z,type="raw"))
	  Yec[,,kcas]<-as.matrix(predict(dd.nn,X,type="raw"))
	  errc[kcas]<-1-mean(apply((Y[samp,]-as.matrix(Yec[samp,,kcas]))^2,2,mean)/sd(Y)^2)
	  errv[kcas]<-1-mean(apply((Y[-samp,]-as.matrix(Yec[-samp,,kcas]))^2,2,mean)/sd(Y)^2)
	  print(c(kc,kcas,errc[kcas],errv[kcas]),digits=3)
	  for (j in 1:mc) {
		  rmse[j,kcas]=(ymax[j]-ymin[j])*(mean((Y[samp,j]-Yec[samp,j,kcas])^2))^0.5
		  r2[j,kcas]=1-mean((Y[samp,j]-Yec[samp,j,kcas])^2)/var(Y[samp,j])
		  rmsep[j,kcas]=(ymax[j]-ymin[j])*(mean((Y[-samp,j]-Yec[-samp,j,kcas])^2))^0.5
		  re[j,kcas]=1-mean((Y[-samp,j]-Yec[-samp,j,kcas])^2)/mean((Y[-samp,j]-mean(Y[samp,j]))^2)
		  ce[j,kcas]=1-mean((Y[-samp,j]-Yec[-samp,j,kcas])^2)/mean((Y[-samp,j]-mean(Y[-samp,j]))^2)
	  }	
	  nns<-nnsens(dd.nn,c(xmin,ymin),c(xmax,ymax),c(colnames(X),colnames(Y)),kcas,ncas,resp=nns)
  }
#plot(fclim[,1],Zec[,1,kcas],col="red",ylab=colnames(fclim)[1],xlab=paste(kcas))
}

if (kcas>10) {


cat("Calibration r2 (median and 95%-conf. interval)\n")
print(quantile(errc,probs=c(0.025, 0.5, 0.975),na.rm=T))
cat("Verification r2 (median and 95%-conf. interval)\n")
print(quantile(errv,probs=c(0.025, 0.5, 0.975),na.rm=T))


# climatic variable loop

cat("In the verification step, r2p (or reduction of error) compares the predictions to the calibration period climatology\n") 
cat("In the verification step, CE (or coefficient of efficiency) compares the predictions to the verification period climatology\n") 


Ye3<-apply(Yec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975),na.rm=T)
Ze3<-apply(Zec,c(1,2),quantile,probs=c(0.025, 0.5, 0.975),na.rm=T)

for (j in 1:mc) {

# calibration
	Ye3[,,j]<-Ye3[,,j]*(ymax[j]-ymin[j])+ymin[j]
	cat("\nDependent Variable : ",colnames(clim[j]),"\n")
	cat("----------------------------------------\n")
	cat("BOOTSTRAP SUMMARY\n")
	ANN$rmseb[j,]=quantile(rmse[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
	cat("bootstrap rmse (median and 95%-conf. interval): ",ANN$rmseb[j,],"\n")
	ANN$rmsepb[j,]=quantile(rmsep[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
	cat("bootstrap rmsep (median and 95%-conf. interval): ",ANN$rmsepb[j,],"\n")
	ANN$r2b[j,]=quantile(r2[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
	cat("bootstrap r2 (median and 95%-conf. interval): ",ANN$r2b[j,],"\n")
	ANN$reb[j,]=quantile(re[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
	cat("bootstrap re (median and 95%-conf. interval): ",ANN$reb[j,],"\n")
	ANN$ceb[j,]=quantile(ce[j,],probs=c(0.025, 0.5, 0.975),na.rm=T)
	cat("bootstrap ce (median and 95%-conf. interval): ",ANN$ceb[j,],"\n")

	ANN$clim[,(j-1)*3+1]<-Ye3[2,,j]
	ANN$clim[,(j-1)*3+2]<-Ye3[2,,j]-Ye3[1,,j]
	ANN$clim[,(j-1)*3+3]<-Ye3[3,,j]-Ye3[2,,j]
	ANN$rmse[j]<-mean((clim[,j]-Ye3[2,,j])^2)^0.5
	ANN$r2[j]<-1-mean((clim[,j]-Ye3[2,,j])^2)/var(clim[,j])
	cat("TOTAL SUMMARY (all observations) \n")
	cat("R2 = ",ANN$r2[j],"  -- RMSE = ",ANN$rmse[j],"calculated on ",na,"  observations\n")
	smin=min(clim[,j])
	smax=max(clim[,j])
	plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(clim)[j],'calib'), ylab="Predicted ANN", main="",col="green")
	points(clim[,j],ANN$clim[,(j-1)*3+1],pch=19,col='blue')
# verification

	if (option=="verification")
		{
		Ze3[,,j]<-Ze3[,,j]*(ymax[j]-ymin[j])+ymin[j]
		ANN$rmsep[j]<-mean((fclim[,j]-Ze3[2,,j])^2)^0.5
		ANN$r2p[j]<-1-mean((fclim[,j]-Ze3[2,,j])^2)/var(clim[,j])
		ANN$CE[j]<-1-mean((fclim[,j]-Ze3[2,,j])^2)/mean((clim[,j]-mean(fclim[,j]))^2)
		cat("R2p (RE) = ",ANN$r2p[j],"  -- CE = ",ANN$CE[j],"  -- RMSEp = ",ANN$rmsep[j],"calculated on ",nfa,"  observations\n")
		ANN$fclim[,(j-1)*3+1]<-Ze3[2,,j]
		ANN$fclim[,(j-1)*3+2]<-Ze3[2,,j]-Ze3[1,,j]
		ANN$fclim[,(j-1)*3+3]<-Ze3[3,,j]-Ze3[2,,j]
		smin<-min(cbind(t(clim[,j]),t(fclim[,j])))
		smax<-max(cbind(t(clim[,j]),t(fclim[,j])))
		plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(clim)[j],' calib:blue/verif:red'), ylab="Predicted ANN", main="",col="green")
		points(clim[,j],ANN$clim[,(j-1)*3+1],pch=19,col='blue')
		points(fclim[,j],ANN$fclim[,(j-1)*3+1],col='red',pch=21)
		}
	else
		{

# extrapolation
		cat("the estimated climate is put in the ANN$fclim matrix\n")
		Ze3[,,j]<-Ze3[,,j]*(ymax[j]-ymin[j])+ymin[j]
		ANN$fclim[,(j-1)*3+1]<-Ze3[2,,j]
		ANN$fclim[,(j-1)*3+2]<-Ze3[2,,j]-Ze3[1,,j]
		ANN$fclim[,(j-1)*3+3]<-Ze3[3,,j]-Ze3[2,,j]
		}
	}

coln<-rep(NA,mc*3)
for (j in 1:mc) {
	coln[(j-1)*3+1]<-colnames(clim)[j]
	coln[(j-1)*3+2]<-paste(colnames(clim)[j],'_i',sep="")
	coln[(j-1)*3+3]<-paste(colnames(clim)[j],'_s',sep="")
}
colnames(ANN$clim)<-coln
colnames(ANN$fclim)<-coln
rownames(ANN$clim)<-rownames(clim)
rownames(ANN$fclim)<-rownames(fass)	
if (!is.null(outfile)) {
	sink(file=ftext,append=TRUE)
	cat("\nImpression de la liste ANN\n")
	print(ANN)
	dev.off()
	sink()
	sink()
	sink()
}

} else {
    print('Too few cases retained')
    ANN=NA
}
return(ANN)
}
