recIK <-
function(ass,clim,fass,fclim=FALSE,p=3,outfile=NULL)
{

# Initialisation
require(MASS)

na<-nrow(ass)
ma<-ncol(ass)
nc<-nrow(clim)
mc<-ncol(clim)
mc3=mc*3
nfa<-nrow(fass)
mfa<-ncol(fass)
if (identical(na,nc)==FALSE) {stop ("both ass & clim matrices have not same number of rows")}
if (identical(mfa,ma)==FALSE) {stop ("both ass & fass matrices have not same number of columns")}
nfc<-0
mfc<-0
option="extrapolation"
IK<-list(title="Transfer function estimates by Imbrie & Kipp method")
IK$rmse<-rep(0,mc)
IK$r2<-rep(0,mc)
IK$clim<-matrix(0,nc,mc3)
IK$fclim<-matrix(0,nfa,mc3)
if (is.logical(fclim)==FALSE) {
	nfc<-nrow(fclim)
	mfc<-ncol(fclim)
	if (identical(nfc,nfa)==FALSE) {stop ("both fclim & fass matrices have not same number of rows")}
	option="verification"
	IK$rmsep<-rep(0,mc)
	IK$r2p<-rep(0,mc)
	IK$CE<-rep(0,mc)
	}

if (!is.null(outfile)) {
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
}
cat("TRANSFER FUNCTION : Imbrie & Kipp method\n")
cat("-----------------------------------------------------------------\n")
cat("\nOption : ",option,"\n")

# transformation in pc

pc.ass<-prcomp(ass,scale.=TRUE)
screeplot(pc.ass, npcs=p, type="barplot",main="Screeplot of the eignevalues")
biplot(pc.ass)
z.ass<-predict(pc.ass,ass)
z.fass<-predict(pc.ass,fass)

# climatic variable loop

cat("In the verification step, r2p (or reduction of error) compares the predictions to the calibration period climatology\n") 
cat("In the verification step, CE (or coefficient of efficiency) compares the predictions to the verification period climatology\n") 

for (j in 1:mc)
	{
# calibration
	cat("\nDependent Variable : ",colnames(clim[j]),"\n")
	cat("----------------------------------------\n")
	mat<-data.frame(cbind(z.ass[1:nc,1:p],vd=clim[1:nc,j]))
	fmla<-as.formula("vd ~ .^2")
	ikm<-lm(fmla, data=mat)
	print(summary(ikm))
	mat<-data.frame(z.ass[,1:p])
	yp<-predict(ikm,mat,se.fit=TRUE)
	IK$clim[,(j-1)*3+1]<-yp$fit
	IK$rmse[j]<-mean((clim[,j]-yp$fit)^2)^0.5
	IK$clim[,(j-1)*3+2]<-IK$rmse[j]
	IK$clim[,(j-1)*3+3]<-IK$rmse[j]
	IK$r2[j]<-1-mean((clim[,j]-yp$fit)^2)/var(clim[,j])
	cat("R2 = ",IK$r2[j],"  -- RMSE = ",IK$rmse[j],"calculated on ",na,"  observations\n")
	smin<-min(clim[,j])
	smax<-max(clim[,j])
	plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(clim)[j],"calib"), ylab="Predicted IK", main="",col="green")
	points(clim[,j],IK$clim[,(j-1)*3+1],pch=19,col='blue')
# verification
	if (option=="verification")
		{
		mat<-data.frame(z.fass[,1:p])
		yp<-predict(ikm,mat,se.fit=TRUE)
		IK$rmsep[j]<-mean((fclim[,j]-yp$fit)^2)^0.5
		IK$r2p[j]<-1-mean((fclim[,j]-yp$fit)^2)/var(fclim[,j])
		IK$CE[j]<-1-mean((fclim[,j]-yp$fit)^2)/var(clim[,j])
		cat("R2p (RE) = ",IK$r2p[j]," -- CE = ",IK$CE[j],"  -- RMSEp = ",IK$rmsep[j],"calculated on ",nfa,"  observations\n")
		IK$fclim[,(j-1)*3+1]<-yp$fit
		IK$fclim[,(j-1)*3+2]<-IK$rmsep[j]
		IK$fclim[,(j-1)*3+3]<-IK$rmsep[j]
		smin<-min(cbind(t(clim[,j]),t(fclim[,j])))
		smax<-max(cbind(t(clim[,j]),t(fclim[,j])))
		plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(clim)[j]," calib (blue) / verif (red)"), ylab="Predicted IK", main="",col="green")
		points(clim[,j],IK$clim[,(j-1)*3+1],pch=19,col='blue')
		points(fclim[,j],IK$fclim[,(j-1)*3+1],col='red',pch=21)
		}
	else
		{

# extrapolation
		cat("the estimated climate is put in the IK$fclim matrix\n")
		matf<-data.frame(z.fass[,1:p])
		yp<-predict(ikm,matf,se.fit=TRUE)
		IK$fclim[,(j-1)*3+1]<-as.matrix(yp$fit)
		IK$fclim[,(j-1)*3+2]<-IK$rmse[j]
		IK$fclim[,(j-1)*3+3]<-IK$rmse[j]
		}
	}
	coln<-rep(NA,mc3)
	for (j in 1:mc) 
		{
		coln[(j-1)*3+1]<-colnames(clim)[j]
		coln[(j-1)*3+2]<-paste(colnames(clim)[j],'_i',sep="")
		coln[(j-1)*3+3]<-paste(colnames(clim)[j],'_s',sep="")
		}
	colnames(IK$clim)<-coln
	colnames(IK$fclim)<-coln
	rownames(IK$clim)<-rownames(clim)
	rownames(IK$fclim)<-rownames(fass)	
	print(IK)
	if (!is.null(outfile)) 
		{
		dev.off()
		sink()
		}
return(IK)
}
