recWAPLS <-
function(ass,clim,fass,fclim=FALSE,p,outfile=NULL)
{
# Initialisation

library(MASS)
library(pls)
na<-nrow(ass)
ma<-ncol(ass)
nc<-nrow(clim)
mc<-ncol(clim)
mc3=mc*3
nfa<-nrow(fass)
mfa<-ncol(fass)
if (identical(na,nc)==FALSE) {stop ("both ass & clim matrices have not same number of rows")}
if (identical(mfa,ma)==FALSE) {stop ("both ass & fass matrices have not same number of columns")}
nfc<-0
mfc<-0
option="extrapolation"
WAPLS<-list(title="Transfer function estimates by WA-PLS method")
WAPLS$rmse<-rep(0,mc)
WAPLS$r2<-rep(0,mc)
WAPLS$clim<-matrix(0,nc,mc3)
WAPLS$fclim<-matrix(0,nfa,mc3)
if (is.logical(fclim)==FALSE) 
	{
	nfc<-nrow(fclim)
	mfc<-ncol(fclim)
	if (identical(nfc,nfa)==FALSE) {stop ("both fclim & fass matrices have not same number of rows")}
	option="verification"
	WAPLS$rmsep<-rep(0,mc)
	WAPLS$r2p<-rep(0,mc)
	WAPLS$CE<-rep(0,mc)
	}
if (!is.null(outfile))
	{
	fgraph=paste(outfile,".pdf",sep="")
	ftext=paste(outfile,".txt",sep="")
	sink(file=ftext)
	pdf(file=fgraph)
	}

cat("TRANSFER FUNCTION : Weighted Averaging Partial Least Squares (WA-PLS) method\n")
cat("------------------------------------------------------------------------------------------------\n")
cat("\nOption : ",option,"\n")

# pre-transformation to make the analysis equivalent to a WAPLS
sit<-apply(ass,1,sum)
fsit<-apply(fass,1,sum)
spe<-apply(ass,2,sum)
z.ass<-diag(sit^-0.5,na,na) %*% as.matrix(ass) %*% diag(spe^-0.5,ma,ma)
rownames(z.ass)<-rownames(ass)
colnames(z.ass)<-colnames(ass)
z.fass<-diag(fsit^-0.5,nfa,nfa) %*% as.matrix(fass) %*% diag(spe^-0.5,ma,ma)
rownames(z.fass)<-rownames(fass)
colnames(z.fass)<-colnames(fass)
climmean=apply(clim,2,mean)
z.clim<-diag(sit^0.5,na,na) %*% scale(clim,center=climmean,scale=FALSE)
rownames(z.clim)<-rownames(clim)
colnames(z.clim)<-colnames(clim)

# climatic variable loop

cat("In the verification step, r2p (or reduction of error) compares the predictions to the calibration period climatology\n") 
cat("In the verification step, CE (or coefficient of efficiency) compares the predictions to the verification period climatology\n") 

for (j in 1:mc)
	{
# calibration
	cat("\nDependent Variable : ",colnames(clim[j]),"\n")
	cat("----------------------------------------\n")
	mat<-data.frame(cbind(z.ass,vd=z.clim[,j]))
	fmla<-as.formula("vd ~ .")
	waplsm<-mvr(fmla,ncomp=p,data=mat,method="kernelpls",validation="LOO")
	print(summary(waplsm))
	ps<-waplsm$validation$ncomp
	cat("Nb components retained = ",ps,"\n")
	mat<-data.frame(z.ass)
	yp=diag(sit^-0.5,na,na)%*%waplsm$fitted.values[,1,ps]+climmean[j]
	WAPLS$clim[,(j-1)*3+1]<-yp[,1]
	WAPLS$rmse[j]<-mean((clim[,j]-yp[,1])^2)^0.5
	WAPLS$clim[,(j-1)*3+2]<-WAPLS$rmse[j]
	WAPLS$clim[,(j-1)*3+3]<-WAPLS$rmse[j]
	WAPLS$r2[j]<-1-mean((clim[,j]-yp[,1])^2)/var(clim[,j])
	cat("R2 = ",WAPLS$r2[j],"  -- RMSE = ",WAPLS$rmse[j],"calculated on ",na,"  observations\n")
	smin<-min(clim[,j])
	smax<-max(clim[,j])
	plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(clim)[j],"calib"), ylab="Predicted WAPLS", main="",col="green")
	points(clim[,j],WAPLS$clim[,(j-1)*3+1],pch=19,col='blue')
# verification
	if (option=="verification")
		{
#		yp<-as.matrix(z.fass)%*%as.matrix(waplsm$coefficients[,1,ps])+waplsm$Ymeans[1]		
		yp=predict.mvr(waplsm,newdata=z.fass,ncomp=ps)
		yp<-diag(sit^-0.5,nfa,nfa)%*%yp+climmean[j]
		WAPLS$rmsep[j]<-mean((fclim[,j]-yp[,1])^2)^0.5
		WAPLS$r2p[j]<-1-mean((fclim[,j]-yp[,1])^2)/var(fclim[,j])
		WAPLS$CE[j]<-1-mean((fclim[,j]-yp[,1])^2)/var(clim[,j])
		cat("R2p (RE) = ",WAPLS$r2p[j],"  -- CE = ",WAPLS$CE[j],"  -- RMSEp = ",WAPLS$rmsep[j],"calculated on ",nfa,"  observations\n")
		WAPLS$fclim[,(j-1)*3+1]<-yp[,1]
		WAPLS$fclim[,(j-1)*3+2]<-WAPLS$rmsep[j]
		WAPLS$fclim[,(j-1)*3+3]<-WAPLS$rmsep[j]
		smin<-min(cbind(t(clim[,j]),t(fclim[,j])))
		smax<-max(cbind(t(clim[,j]),t(fclim[,j])))
		plot(c(smin,smax),c(smin,smax), type='l',xlab=paste(colnames(clim)[j],"calib (blue) / verif (red)"), ylab="Predicted WAPLS", main="",col="green")
		points(clim[,j],WAPLS$clim[,(j-1)*3+1],pch=19,col='blue')
		points(fclim[,j],WAPLS$fclim[,(j-1)*3+1],col='red',pch=21)
		}
	else
		{
# extrapolation
		cat("the estimated climate is put in the WAPLS$fclim matrix\n")
#		yp<-as.matrix(z.fass)%*%as.matrix(waplsm$coefficients[,1,ps])+waplsm$Ymeans[1]		
		yp=predict.mvr(waplsm,newdata=z.fass,ncomp=ps)
		yp<-diag(sit^-0.5,nfa,nfa)%*%yp+climmean[j]
		WAPLS$fclim[,(j-1)*3+1]<-yp[,1]
		WAPLS$fclim[,(j-1)*3+2]<-WAPLS$rmse[j]
		WAPLS$fclim[,(j-1)*3+3]<-WAPLS$rmse[j]
		}
	}
	coln<-rep(NA,mc3)
	for (j in 1:mc) {
		coln[(j-1)*3+1]<-colnames(clim)[j]
		coln[(j-1)*3+2]<-paste(colnames(clim)[j],'_i',sep="")
		coln[(j-1)*3+3]<-paste(colnames(clim)[j],'_s',sep="")
	}
	colnames(WAPLS$clim)<-coln
	colnames(WAPLS$fclim)<-coln
	rownames(WAPLS$clim)<-rownames(clim)
	rownames(WAPLS$fclim)<-rownames(fass)	
	cat("\nImpression de la liste WAPLS\n")
	print(WAPLS)
	if (!is.null(outfile)) 
		{
		dev.off()
		sink()
		}
return(WAPLS)
}
