\name{recGLM}
\alias{recGLM}
\title{ transfer function using the GLM method }
\description{
Calculate a generalized linear model between environmental variables; three options are possible: focus on (M) climates close to their mean values, (H) climates close to their highest values, (L) climates close to their lowest values; regression coefficients are used for prediction.
}
\usage{
output=recGLM(xmat,ymat,xnewmat,ynewmat=FALSE,focus="M",outfile=NULL)
}
\arguments{
  \item{xmat}{ \code{xmat} is the matrix of the predictors (assemblages) }
  \item{ymat}{ \code{ymat} is the matrix of the predictands (environment) }
  \item{xnewmat}{ \code{xnewmat} is the assemblages matrix  for verification or prediction }
  \item{ynewmat}{ \code{ynewmat} is the environmental matrix corresponding to xnewmat, if verification is wanted; if it is a prediction (reconstruction), assign to FALSE }
  \item{focus}{ \code{focus} is the type of values on which the regression will focus (if not assign, it is assumed to be "M")}
  \item{outfile}{ \code{outfile} is the filename for the log and graphics }
}
\details{
The script uses "glm" of library "MASS" for GLM modelling and "prcomp" for the principal components. Each environmental variable is processed independently. The observations are weighted according to the following equation: w<-(clim[,j]-cst)^-2/sum(w) where clim[,j] is the jth climatic variable and cst is defined as follows
(1) if (focus[j]=="L") {cst<-min(clim[,j])*1.01}
(2) if (focus[j]=="H") {cst<-max(clim[,j])*1.01}
(3) if (focus[j]=="M") {cst<-mean(clim[,j])}
The error distribution chosen is gaussian.
}
\value{
output list contains: 
  rmse, r2, calculated on the calibration data; 
  clim, the estimates (with 95% confidence intreval) of the predictands used for calibration;
  fclim the estimates of the new environmental matrix; 
  if there is verification, it contains also: 
    rmsep, RE (or r2p) and CE (calculated on the independent data). 
  RE is the variance explained as compared with the calibration average and CE is the variance explained as compared with the verification average.
}
\references{ ~ ~ }
\author{ J. Guiot }
\note{ void
}
\examples{
output=recGLM(ss700po,ss700cl,fospo%,focus="M")
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ multivariate statistics} 
\keyword{ generalized linear model} 
\keyword{ error weighting}
\keyword{ transfer function} 
\keyword{ GLM}

