\name{recMAT}
\alias{recMAT}
\title{ transfer function using the modern analogue method, eventually with taxon weighting and/or categorial constraints }
\description{
Estimate environment from a modern analogue dataset: the assemblages to predict are compared to modern assemblages using a distance (Euclidian, Chord, log or Camberra), the closest modern assemblages are selected and their environmental variables are averaged to provide the environment wanted. It is possible to weight the taxa to emphasize some of them and it is possible also to select the analogues which correspond to a given criterian. 
}
\usage{
output=recMAT(xmat,ymat,xnewmat,ynewmat=FALSE,ichd=0,kana=8,wghts=NULL,mcon=NULL,fcon=NULL,outfile=NULL)
}
\arguments{
  \item{xmat}{ \code{xmat} is the matrix of the predictors (assemblages) }
  \item{ymat}{ \code{ymat} is the matrix of the predictands (environment) }
  \item{xnewmat}{ \code{xnewmat} is the assemblages matrix  for verification or prediction }
  \item{ynewmat}{ \code{ynewmat} is the environmental matrix corresponding to xnewmat, if verification is wanted; if it is a prediction (reconstruction), assign to FALSE }
  \item{ichd}{ \code{ichd} is the option for the distance (0=euclidian /1=chord /2=log /3=canberra); default = 0}
  \item{kana}{ \code{kana} is the maximum number of analogues; default = 0}
  \item{wghts}{ \code{wghts} is the vector of the weights attributed to the taxa, with a dimension equal to the number of columns of xmat; default = NULL}
  \item{mcon}{ \code{mcon} is the vector of the categories (for example biomes) attributed to the modern assemblages, with a dimension equal to the number of rows of xmat; default = NULL}
  \item{fcon}{ \code{fcon} is the vector of the categories (for example biomes) attributed to the fossil assemblages, with a dimension equal to the number of rows of xnewmat; default = NULL}
  \item{outfile}{ \code{outfile} is the filename for the log and graphics }
}
\details{
    An analogue is a modern assemblage with a distance to the assemblage to estimate lower than a threshold, calculated by Monte-Carlo technique; the kana with the lowest distance are kept: 
    - the chord distance is the Euclidian distance on the taxa transformed by square-root; 
    - the log-distance is the Eurclidian distance on the taxa transformed by natural logarithm;
    - the Camberra distance between x and y is given by sum(|x_i - y_i| / |x_i + y_i|)

Taxon weighting:
If wghts is not NULL, it contains a coefficient vector: the jth coefficient is the weight used to multiply the jth taxon in the distance summation.

Categorial constraints
If mcon and fcon are not NULL, they are vectors of alphanumerical strings representing a category (e.g. a biome) defining a type for each assemblage. mcon must have the same dimension than the first dimension of xmat and fcon must have the same dimension than the first dimension of xnewmat. Only the analogues with a category matching with that of the assemblage to estimate are kept.

The reconstructed environment is the average of the environment of the analogues, weighted by the inverse distance.
}
\value{
output list contains: 
  rmse, r2, calculated on the calibration data; 
  clim, the estimates (with 95% confidence intreval) of the predictands used for calibration;
  fclim the estimates of the new environmental matrix; 
  if there is verification, it contains also: 
    rmsep, RE (or r2p) and CE (calculated on the independent data). 
  RE is the variance explained as compared with the calibration average and CE is the variance explained as compared with the verification average.
}
\references{ ~ ~ }
\author{ J. Guiot }
\note{ void
}
\examples{
output=recMAT(ss700po,ss700cl,fospo%,ichord=1,kana=8)
}

\keyword{ multivariate statistics}
\keyword{ analogues}
\keyword{ weighted average} 
\keyword{ transfer function}

