\name{wavecohsp}
\alias{wavecohsp}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ calculates FFT co-spectrum of a multivariate time-series }
\description{
  Transform the x matrix (m time-series) into wavelet  cospectra using the bivar2 package   
}
\usage{
output=wavecohsp(data,s0=2,noctave=6,nvoice=20,w0=6,sw=0,siglevel=0.95,nreal=100,logscale=F,units="",outfile=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{ \code{data} is the input matrix, dim (n,m) }
  \item{s0}{ \code{s0} is the Nyquist period }
  \item{noctave}{ \code{noctave} is the number of octaves, i.e. of powers of 2 to define in the frequency scale: 2^(0:(noctave))*s0 }
  \item{nvoice}{ \code{nvoice} is the number of voices per octaves, the total number of frequencies resolved is noctave*nvoice+1  }
  \item{w0}{time/frequency resolution omega0}
  \item{sw}{ \code{sw} is the number of one-side weights to smooth the wavelet transform using filter function; total number of weights is 2*sw+1; 0 means no filtering  }
  \item{siglevel}{ \code{siglevel} is the confidence level to be used for tests; defaults = 0.95 }
  \item{nreal}{ \code{nreal} is the number of realizations for MC simulations used for significance tests; default = 100 }
  \item{logscale}{ \code{logscale} is TRUE when log-scale is wanted; default = FALSE }
  \item{units}{ \code{units} is the data units to add to the graphics }
  \item{outfile}{ \code{outfile} is the filename for the log and graphics }
}
  \details{
This function calculate the Morley wavelet transform to perform a time-dependent cospectral analysis. It uses the package bivar of D. Maraun.  
  The default of sw is set to zero to avoid uncritical use of the
  function. One should always play a bit with this value to find out how
  critical the result depends on it. Choosing different values for sw
  also gives an impression of which peaks might be spurious, because
  they only appear for special choices of sw.
  The cone of influence is marked by black lines. Values outside the
  cone of influence should be interpreted very carefully, as they result
  from a significant contribution of zero padding at the beginning and
  the end of the time series.
  For a better visualization, additional dotted
  lines marking distinct times or scales might be plotted by providing
  the vectors markt and marks.


}
\value{
  output is a list wclist of components
  \item{coherency}{Matrix of wavelet coherency of dimension
    [length(series)]x[nvoice*noctave+1]}
  \item{phase}{Matrix of phase of wavelet cross spectrum, dimension as coherency.}
  \item{times}{Vector of times of length(series)}
  \item{scales}{Vector of scales of length nvoice*noctave+1}
  \item{critval}{The critical value for the chosen significance level}
}
\references{ D. Maraun and J. Kurths, Nonlin. Proc. Geophys. 11:
  505-514, 2004 }
\author{ D. Maraun}
\note{ void
}
\examples{
cs=wavecohsp(globclim)
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ multivariate time-series analysis }
\keyword{ cospectral analysis }
\keyword{ wavelet analysis }